package br.usp.ime.algorithms;

import java.util.ArrayList;
import java.util.List;
import java.util.PriorityQueue;

import br.usp.ime.evolvinggraph.Edge;
import br.usp.ime.evolvinggraph.EvolvingGraph;
import br.usp.ime.evolvinggraph.Journey;
import br.usp.ime.evolvinggraph.Node;
import br.usp.ime.priorityqueueelement.ForemostElement;

public class ForemostAlgorithm implements Algorithm{
	
	final Integer INFINITY = Integer.MAX_VALUE;
	List<ForemostElement> heapElements;
	
	public List<Journey> calculate(EvolvingGraph graph, Node s) {
		return calculate(graph, s, 0);
	}
	
	public List<Journey> calculate(EvolvingGraph graph, Node s, int initTime) {
		int n = graph.getNumOfNodes();
		List<Journey> foremostJourneys = new ArrayList<Journey>(n);
		PriorityQueue<ForemostElement> heap = new PriorityQueue<ForemostElement>();
		heapElements = new ArrayList<ForemostElement>();
		
		for (int i = 0; i < n; i++){
			ForemostElement element = new ForemostElement(graph.getNode(i), INFINITY);
			heap.add(element);
			heapElements.add(element);
			foremostJourneys.add(new Journey());
		}
		
		ForemostElement elementS = getForemostElement(s);
		heap.remove(elementS);
		elementS.setValue(initTime);
		heap.add(elementS);
		
		while (heap.isEmpty() == false){
			ForemostElement elementU = heap.remove();
			
			if (elementU.getValue() == INFINITY) break;
			Node u = (Node) elementU.getObject();
			
			for(Edge edge : u.getAdj()){
				Node v = edge.getV();
				ForemostElement elementV = getForemostElement(v);
				Integer t = edge.getNextDeparture(elementU.getValue());
				if (t != null) {
					if (t + edge.getCost() < elementV.getValue()) {
						heap.remove(elementV);
						elementV.setValue(t + edge.getCost());
						Journey journey = new Journey(foremostJourneys.get(u.getId()), edge, t);
						foremostJourneys.remove(v.getId());
						foremostJourneys.add(v.getId(), journey);
						heap.add(elementV);
					}
				}
			}
		}
		
		return foremostJourneys;
	}

	private ForemostElement getForemostElement(Node node) {
		ForemostElement e = heapElements.get(node.getId());
		return e;
	}
	
}