#!/usr/bin/perl -w

package Toolkit;
use strict;
use warnings;
use Getopt::Long;
use FileHandle;

#forma de rodar o script: ./generate.pl NOMEDOALGORITMO EIXOX
#NOMEDOALGORITMO pode ser: Shortest, Foremost, Fastest, Epidemic, Prophet, MaxProp
#EIXOX pode ser: raio, buffer, nos , mensagens

use constant NAME       => 0;
use constant END_TM     => 3;
use constant ROUTER     => 7;
use constant RANGE      => 8;
use constant TSPEED     => 9;
use constant NODES      => 12;
use constant BUFFER     => 13;
use constant SPEED      => 14;
use constant MSG_SIZE   => 20;
use constant HOST		=> 22;
use constant MOV_SEED   => 24;

use constant NUMBER    => 10;
	
#variaveis globais
my $router;
my @array_conf;
my $xmin;
my $xmax;
my $in;
my $act;
my $var_line;

#variaveis passadas na entrada
my $scenarioName = $ARGV[0];
my $xaxis = $ARGV[1];

#Determina qual o router será usado:
	if(lc($scenarioName) eq "shortest"){
		$router = "ShortestRouter";
	}
	elsif(lc($scenarioName) eq "foremost"){
		$router = "ForemostRouter";
	}
	elsif(lc($scenarioName) eq "fastest"){
		$router = "FastestRouter";
	}
	elsif(lc($scenarioName) eq "epidemic"){
		$router = "EpidemicRouter";
	}
	elsif(lc($scenarioName) eq "prophet"){
		$router = "ProphetRouter";
	}
	elsif(lc($scenarioName) eq "maxprop"){
		$router = "MaxPropRouter";
	}
    else{
        die "Unknown router";
    }
	
#Cria arquivo de configurações baseado no comparison_settings
	open (CONF, "../comparison_settings.txt") or die "Couldn't open file comparison_settings.txt";
	@array_conf = <CONF>;
	close (CONF);
	open (TEMP, ">../temp_settings.txt") or die "Couldn't open the configuration file temp_settings.txt";
	print TEMP @array_conf;
	close (TEMP);		
	
#Altera o arquivo comparision_settings para o ROUTER a ser usado
	open (CONF, "../temp_settings.txt") or die "Couldn't open file temp_settings.txt";
	@array_conf = <CONF>;
	close (CONF);
	open (CONF, ">../temp_settings.txt") or die "Couldn't open file temp_settings.txt";
	$array_conf[NAME] = "Scenario.name = $scenarioName\n";
	$array_conf[ROUTER] = "Group.router = $router\n";
	print CONF @array_conf;
	close (CONF);
	
#Cria o arquivo de saída dependendo do ROUTER

	open (CONF, ">../results/".$scenarioName."_".$xaxis."_Relayed") or die "Couldn't open file ../results/".$scenarioName."_".$xaxis."_Relayed";
	close(CONF);
	open (CONF, ">../results/".$scenarioName."_".$xaxis."_Delivered") or die "Couldn't open file";
	close(CONF);
	open (CONF, ">../results/".$scenarioName."_".$xaxis."_Latency") or die "Couldn't open file ";
	close(CONF);
	open (CONF, ">../results/".$scenarioName."_".$xaxis."_Hopcount") or die "Couldn't open file ";
	close(CONF);
	open (CONF, ">../results/".$scenarioName."_".$xaxis."_TransitTime") or die "Couldn't open file ";
	close(CONF);
	if($scenarioName eq "shortest" || $scenarioName eq "foremost" || $scenarioName eq "fastest"){		
		open (CONF, ">../results/".$scenarioName."_".$xaxis."_Time") or die "Couldn't open file ";
		close(CONF);
	}

#Intervalo usado dependendo do eixo x
	if($xaxis eq "raio"){
		$in = 10;
		$xmin = 5;
		$xmax = 105;
	}
	elsif($xaxis eq "buffer"){
		$in = 1;
		$xmin = 1;
		$xmax = 10;
	}
	elsif($xaxis eq "nos"){
		$in = 10;
		$xmin = 50;
		$xmax = 150;
	}	
	elsif($xaxis eq "mensagens"){
		$in = 50;
		$xmin = 50;
		$xmax = 500;
	}
		
#Simulações serão feitas adiante:
	$act = $xmin;

	while($act <= $xmax){
		my $relayed;
  		my $delivered;
   		my $latency;
		my $transittime;
    	my $hopcount;
		my $usertime;
		my $maxrelayed = 0;
    	my $maxdelivered = 0;
    	my $maxlatency = 0;
		my $maxtransittime = 0;
    	my $maxhopcount = 0;
		my $maxtime = 0;
    	my $minrelayed = 10000000;
    	my $mindelivered = 10000000;
    	my $minlatency = 10000000;
    	my $mintransittime = 10000000;
    	my $minhopcount = 10000000;
		my $mintime = 10000000;
    	my $totalrelayed = 0;
    	my $totaldelivered = 0;
    	my $totallatency = 0;
    	my $totaltransittime = 0;
    	my $totalhopcount = 0;
		my $totaltime = 0;
			
		my $count = 0;
		
    	if($xaxis eq "raio"){
   	 		$array_conf[RANGE] = "Group.transmitRange = $act\n";
    	}
  		elsif($xaxis eq "buffer"){
        	$array_conf[BUFFER] = "Group1.bufferSize = $act"."M\n";
    	}
	   	elsif($xaxis eq "nos"){
        	$array_conf[NODES] = "Group1.nrofHosts = $act\n";
        	$act--;
        	$array_conf[HOST] =	"Events1.hosts = 0,$act\n";
        	$act++;
    	}
 	  		elsif($xaxis eq "mensagens"){
        	$array_conf[MSG_SIZE] = "Events1.size = $act"."k\n";
    	}
        else {
            print "Invalid xaxis: $xaxis";
        }
		
		while($count < NUMBER){
     	    
     	    open (CONF, ">../temp_settings.txt") or die "Couldn't open file temp_settings.txt";    
            my $seed = $count;
            $array_conf[MOV_SEED] = "MovementModel.rngSeed = $seed\n";
    		print CONF @array_conf;
	    	close (CONF);
		
			#se o router usado for o SHORTEST ou o FOREMOST
			if($scenarioName eq "shortest" || $scenarioName eq "foremost" || $scenarioName eq "fastest"){		
				system("cd ..;sh one6.sh -b 1 temp_settings.txt");
				if($xaxis eq "nos"){
					# system("cd ..; cd ..; cd TestedEvolvingGraphs ; ant OneIntegration -Dargs=\"$act 3000 $scenarioName\" ");
					open(INTEGRATION, "cd ..; cd ..; cd TestedEvolvingGraphs ; /usr/bin/time ant OneIntegration -Dargs=\"$act 3000 $scenarioName\" 2>&1 |");
				}
				else{
					# system("cd ..; cd ..; cd TestedEvolvingGraphs ; ant OneIntegration -Dargs=\"125 3000 $scenarioName\" ");
					open(INTEGRATION, "cd ..; cd ..; cd TestedEvolvingGraphs ; /usr/bin/time ant OneIntegration -Dargs=\"125 3000 $scenarioName\" 2>&1 |");
				}
				while (my $integOut = <INTEGRATION>){
					print $integOut;
					if ($integOut =~ m/^(\d+.\d+)user/){
						print "UTime $1\t";
						$usertime = $1;
						if ($usertime < $mintime){
							$mintime = $usertime;
						}
						if ($usertime > $maxtime){
							$maxtime = $usertime;
						}
						$totaltime += $usertime;
					}
				}
				close(INTEGRATION);
             	system("cd ..; ant");
				system("cd ..;sh one6.sh -b 1 temp_settings.txt")
			}
			else{
				#one6 é pra funcionar na rede linux.. :p
				system("cd ..;sh one6.sh -b 1 temp_settings.txt");
				#system("cd ..;sh one.sh -b 1 temp_settings.txt");
			}	
		
			my $messagesReport = $scenarioName."_MessageStatsReport.txt";
			
			#Aqui devem ser inseridos os tipos de possiveis variaveis no eixo y, aquelas do REPORT
			
			open(FILE, "../reports/test/".$messagesReport) or die "Couldn't open the $messagesReport";
				while(my $lineFromMessages = <FILE>) {
					#relayed
					if($lineFromMessages =~ m/^relayed: (\d+)/){
						$relayed = $1;
						print "Relay $1\t";
						if($relayed < $minrelayed){
							$minrelayed = $relayed;
						}
						if($relayed > $maxrelayed){
							$maxrelayed = $relayed;
						}
						$totalrelayed += $relayed;
						#TODO media, min e max
                       	#print "raio: $act \nprob: $deliveredMessages .$messages\n";
					}
					if($lineFromMessages =~ m/^delivery_prob: (\d+.\d+)/){
						print "DelPr $1\t";
						$delivered = $1;
						if($delivered < $mindelivered){
							$mindelivered = $delivered;
						}
						if($delivered > $maxdelivered){
							$maxdelivered = $delivered;
						}
						
						$totaldelivered += $delivered;
                       	#TODO media, min e max
                       	#print "raio: $act \nprob: $deliveredMessages .$messages\n";
					}
					if($lineFromMessages =~ m/^latency_avg: (\d+.\d+)/){
						print "Ltncy $1\t";
						$latency = $1;
						if($latency < $minlatency){
							$minlatency = $latency;
						}
						if($latency > $maxlatency){
							$maxlatency = $latency;
						}
						$totallatency += $latency;
                       	#TODO media, min e max
                       	#print "raio: $act \nprob: $deliveredMessages .$messages\n";
					}
					if($lineFromMessages =~ m/^transit_time_avg: (\d+.\d+)/){
						print "TTime $1\t";
						$transittime = $1;
						if($transittime < $mintransittime){
							$mintransittime = $transittime;
						}
						if($transittime > $maxtransittime){
							$maxtransittime = $transittime;
						}
						$totaltransittime += $transittime;
                       	#TODO media, min e max
                       	#print "raio: $act \nprob: $deliveredMessages .$messages\n";
					}
					if($lineFromMessages =~ m/^hopcount_avg: (\d+.\d+)/){
						print "Hopct $1\n";
						$hopcount = $1;
						if($hopcount < $minhopcount){
							$minhopcount = $hopcount;
						}
						if($hopcount > $maxhopcount){
							$maxhopcount = $hopcount;
						}
						$totalhopcount += $hopcount;
                       	#TODO media, min e max
                   	    #print "raio: $act \nprob: $deliveredMessages .$messages\n";
					}
				}
			close(FILE);
			$count++;		
			
		}
		###########################################################################################
		#TODO Aqui deve ser alterado para incluir as médias, o máximo e o minimo das 30 vezes
		
		my $avg;
		
		open (CONF, ">>../results/".$scenarioName."_".$xaxis."_Relayed") or die "Couldn't open file";
		$avg = $totalrelayed/NUMBER;
		printf CONF "%.2f %.2f %.2f %.2f\n", $act, $avg, $minrelayed, $maxrelayed ;
		close(CONF);
		
		open (CONF, ">>../results/".$scenarioName."_".$xaxis."_Delivered") or die "Couldn't open file";
		$avg = $totaldelivered/NUMBER;
		printf CONF "%.2f %.2f %.2f %.2f\n", $act, $avg, $mindelivered, $maxdelivered ;
		close(CONF);
		
		open (CONF, ">>../results/".$scenarioName."_".$xaxis."_Latency") or die "Couldn't open file ";
		$avg = $totallatency/NUMBER;
		printf CONF "%.2f %.2f %.2f %.2f\n", $act, $avg, $minlatency, $maxlatency ;
		close(CONF);

		open (CONF, ">>../results/".$scenarioName."_".$xaxis."_TransitTime") or die "Couldn't open file ";
		$avg = $totaltransittime/NUMBER;
		printf CONF "%.2f %.2f %.2f %.2f\n", $act, $avg, $mintransittime, $maxtransittime ;
		close(CONF);
		
		open (CONF, ">>../results/".$scenarioName."_".$xaxis."_Hopcount") or die "Couldn't open file ";
		$avg = $totalhopcount/NUMBER;
		printf CONF "%.2f %.2f %.2f %.2f\n", $act, $avg, $minhopcount, $maxhopcount ;
		close(CONF);

		if($scenarioName eq "shortest" || $scenarioName eq "foremost" || $scenarioName eq "fastest"){
			open (CONF, ">>../results/".$scenarioName."_".$xaxis."_Time") or die "Couldn't open file ";
			$avg = $totaltime/NUMBER;
			printf CONF "%.2f %.2f %.2f %.2f\n", $act, $avg, $mintime, $maxtime ;
			close(CONF);
		}
				
		$act += $in;
	}
	
	system("rm -f ../temp_settings.txt");

