#!/usr/bin/perl -w

#This script is based on the configuration file called comparison_settings.txt.
#This script should be located at the toolkit directory (/one_1.3.0/toolkit/).
#The configuration file should be located at the basis directory (/one_1.3.0/).

#Last revision: July 17, 2009.
#Last revisor: Caio Cestari Silva.

package Toolkit;
use strict;
use warnings;
use FileHandle;

our @routingType = ("EpidemicRouter", "ProphetRouter", "MaxPropRouter");
our @routingColor = ("orange", "blue", "black");

our $routersSize = @routingType;
our $scenarioName;
our $routingName;

our $routerLine;
our $intervalLine;
our $transmitLine;
our $hostsLine;

our $outputLine;
our $yRangeLine;
our $xRangeLine;
our $yLabelLine;
our $xLabelLine;
our $titleLine;

our @originalConfigFile;
our @configFile;
our @originalPlotFile;
our @plotFile;

our $deliveredMessagesFile;
our $averageHopsFile;
our $averageTimeFile;
our $delayPerRadiusFile;
our $probabilityPerRadiusFile;
our $delayPerNodesFile;

our $createdMessages;
our $startedMessages;
our $relayedMessages;
our $deliveredMessages;
our $deliveryProbability;
our $averageLatency;
our $averageHopcount;
our $transmitRadius;
our $numberOfHosts;

our $maxDeliveredMessages = 0;
our $maxAverageHopsMessages = 0;
our $maxAverageLatencyMessages = 0;
our $maxAverageLatencyRadius = 0;
our $maxDeliveryProbabilityRadius = 0;
our $maxDeliveryProbabilityHosts = 0;

sub GetFileAttributes() {
	
	my $lineCounter = 0;
	
	open (SETTINGS, "../comparison_settings.txt") or die "Couldn't open the configuration file comparison_settings.txt.";
	@originalConfigFile = <SETTINGS>;
	
	seek(SETTINGS, 0, 0);
	
	while (my $lineFromData = <SETTINGS>) {
		if ($lineFromData =~ m/^Scenario.name = (\w+)/) {
			$scenarioName = $1;
		}
		elsif ($lineFromData =~ m/^Group.router = /) {
			$routerLine = $lineCounter;
		}
		elsif ($lineFromData =~ m/^Events1.interval = /) {
			$intervalLine = $lineCounter;
		}
		elsif ($lineFromData =~ m/^Group.transmitRange = /) {
			$transmitLine = $lineCounter;
		}
		elsif ($lineFromData =~ m/^Group1.nrofHosts = /) {
			$hostsLine = $lineCounter;
		}
		$lineCounter++;
	}
	
	close (SETTINGS);
	
}

sub CreateDataFiles() {
	
	$deliveredMessagesFile = "delivered_messages_".$routingName.".txt";
	open(TEMPFILE, ">".$deliveredMessagesFile) or die "Couldn't create the temp log file ".$deliveredMessagesFile;
	close(TEMPFILE);
	
	$averageHopsFile = "created_avghops_".$routingName.".txt";
	open(TEMPFILE, ">".$averageHopsFile) or die "Couldn't create the temp log file ".$averageHopsFile;
	close(TEMPFILE);
	
	$averageTimeFile = "created_avgtime_".$routingName.".txt";
	open(TEMPFILE, ">".$averageTimeFile) or die "Couldn't create the temp log file ".$averageTimeFile;
	close(TEMPFILE);
	
	$delayPerRadiusFile = "delay_varying_radius_".$routingName.".txt";
	open(TEMPFILE, ">".$delayPerRadiusFile) or die "Couldn't create the temp log file ".$delayPerRadiusFile;
	close(TEMPFILE);
	
	$probabilityPerRadiusFile = "probability_varying_radius_".$routingName.".txt";
	open(TEMPFILE, ">".$probabilityPerRadiusFile) or die "Couldn't create the temp log file ".$probabilityPerRadiusFile;
	close(TEMPFILE);
	
	$delayPerNodesFile = "probability_varying_nodes_".$routingName.".txt";
	open(TEMPFILE, ">".$delayPerNodesFile) or die "Couldn't create the temp log file ".$delayPerNodesFile;
	close(TEMPFILE);
	
}

sub RegisterMessagesGatheredResults() {
	
	open (TEMPFILE, ">>".$deliveredMessagesFile) or die "Couldn't open file ".$deliveredMessagesFile; 
	if($deliveredMessages > $maxDeliveredMessages) {
		$maxDeliveredMessages = $deliveredMessages;
	}
	printf TEMPFILE "%f %f\n", $createdMessages, $deliveredMessages;
	close (TEMPFILE);

	open (TEMPFILE, ">>".$averageHopsFile) or die "Couldn't open file".$averageHopsFile;
	if($averageHopcount > $maxAverageHopsMessages) {
		$maxAverageHopsMessages = $averageHopcount;
	}
	printf TEMPFILE "%f %f\n", $createdMessages, $averageHopcount;
	close (TEMPFILE);

	open (TEMPFILE, ">>".$averageTimeFile) or die "Couldn't open file".$averageTimeFile;
	if($averageLatency > $maxAverageLatencyMessages) {
		$maxAverageLatencyMessages = $averageLatency;
	}
	printf TEMPFILE "%f %f\n", $createdMessages, $averageLatency;
	close (TEMPFILE);
	
}

sub RegisterRadiusGatheredResults() {
	
	open (TEMPFILE, ">>".$delayPerRadiusFile) or die "Couldn't open file ".$delayPerRadiusFile;
	if($maxAverageLatencyRadius > $averageLatency) {
		$maxAverageLatencyRadius = $averageLatency;
	}
	printf TEMPFILE "%f %f\n", $transmitRadius, $averageLatency;
	close (TEMPFILE);
	
	open (TEMPFILE, ">>".$probabilityPerRadiusFile) or die "Couldn't open file ".$probabilityPerRadiusFile; 
	if($maxDeliveryProbabilityRadius > $deliveryProbability) {
		$maxDeliveryProbabilityRadius = $deliveryProbability;
	}
	printf TEMPFILE "%f %f\n", $transmitRadius, $deliveryProbability;
	close (TEMPFILE);
	
}

sub RegisterHostsGatheredResults() {
	
	open (TEMPFILE, ">>".$delayPerNodesFile) or die "Couldn't open file ".$delayPerNodesFile; 
	if($maxDeliveryProbabilityHosts > $deliveryProbability) {
		$maxDeliveryProbabilityHosts = $deliveryProbability;
	}
	printf TEMPFILE "%f %f\n", $numberOfHosts, $deliveryProbability;
	close (TEMPFILE);

}

sub RunSimulationAndGatherResults() {
	
	system("cd ..;sh one.sh -b 1 temp_iteration_settings.txt > simulation_log.txt");
	
	my $lineFromReport;
	
	my $messagesReport = $scenarioName."_MessageStatsReport.txt";
	open(MESSAGESREPORT, "../reports/test/".$messagesReport) or die "Couldn't open the $messagesReport";
	
	while($lineFromReport = <MESSAGESREPORT>) {
		
		if ($lineFromReport =~ m/^created: (\d+)/) {
			$createdMessages = $1;	
		}
		elsif ($lineFromReport =~ m/^started: (\d+)/) {
			$startedMessages = $1;
		}
		elsif ($lineFromReport =~ m/^relayed: (\d+)/) {
			$relayedMessages = $1;
		}
		elsif ($lineFromReport =~ m/^delivered: (\d+)/) {
			$deliveredMessages = $1;
		}
		elsif ($lineFromReport =~ m/^delivery_prob: (\d+)/) {
			$deliveryProbability = $1;
		}
		elsif ($lineFromReport =~ m/^latency_avg: (\d+)/) {
			$averageLatency = $1;
		}		
		elsif ($lineFromReport =~ m/^hopcount_avg: (\d+)/) {
			$averageHopcount = $1;
		}
	
	}
		
}

sub VaryOnMessages() {
	
	my $intervalGenerator;
	
	print "Generating messages every X milisseconds, X belonging to [1, 200], increasing by 5.\n";
	@configFile = @originalConfigFile;
	
	for($intervalGenerator = 1; $intervalGenerator <= 200; $intervalGenerator += 5) {
		
		$configFile[$intervalLine] = "Events1.interval = $intervalGenerator, $intervalGenerator\n";
		open (TEMPSETTINGS, ">../temp_iteration_settings.txt") or die "Couldn't open the configuration file temp_iteration_settings.txt.";
		print TEMPSETTINGS @configFile;
		close (TEMPSETTINGS);
		
		RunSimulationAndGatherResults();
		RegisterMessagesGatheredResults();
		
	}
	
}

sub VaryOnRadius() {
	
	my $radiusGenerator;
	
	print "Increasing radius by 2, starting on 2 until reaching 50.\n";
	@configFile = @originalConfigFile;
	
	for($radiusGenerator = 2; $radiusGenerator <= 50; $radiusGenerator += 2) {
		
		$transmitRadius = $radiusGenerator;
		
		$configFile[$transmitLine] = "Group.transmitRange = $radiusGenerator\n";
		open (TEMPSETTINGS, ">../temp_iteration_settings.txt") or die "Couldn't open the configuration file temp_iteration_settings.txt.";
		print TEMPSETTINGS @configFile;
		close (TEMPSETTINGS);
		
		RunSimulationAndGatherResults();
		RegisterRadiusGatheredResults();
		
	}
	
}

sub VaryOnHosts() {
	
	my $hostsGenerator;
	
	print "Increasing hosts by 2, starting on 5 until reaching 50.\n";
	@configFile = @originalConfigFile;
	
	for($hostsGenerator = 5; $hostsGenerator <= 50; $hostsGenerator += 2) {
		
		$numberOfHosts = $hostsGenerator;
		
		$configFile[$hostsLine] = "Group1.nrofHosts = $hostsGenerator\n";
		open (TEMPSETTINGS, ">../temp_iteration_settings.txt") or die "Couldn't open the configuration file temp_iteration_settings.txt.";
		print TEMPSETTINGS @configFile;
		close (TEMPSETTINGS);
		
		RunSimulationAndGatherResults();
		RegisterHostsGatheredResults();
		
	}
	
}

sub Simulate() {

	print "Router type: ".$routingName."\n";
	print "Simulation began...\n";
	
	VaryOnMessages();
	VaryOnRadius();
	VaryOnHosts();
	
	print "Simulation ended.\n";
	
}

sub IterateUnderRouters() {
	
	my $iterator = 0;
	
	while($iterator < $routersSize)	{
		
		@configFile = @originalConfigFile;
		
		$configFile[$routerLine] = "Group.router = $routingType[$iterator]\n";
		
		open (TEMPSETTINGS, ">../temp_iteration_settings.txt") or die "Couldn't create the configuration file temp_iteration_settings.txt.";
		print TEMPSETTINGS @configFile;
		close (TEMPSETTINGS);
		$routingName = $routingType[$iterator];
		
		CreateDataFiles();
		Simulate();
		
		$iterator++;
		
	}
	
}

sub ClearGarbage() {
	
	system("rm -f ../temp_iteration_settings.txt");
	
}

sub ParsePlottingConfig() {
	
	my $lineCounter = 0;
	
	open(PLOTTING, "config.plot") or die "Couldn't open config.plot for edition.";
	@originalPlotFile = <PLOTTING>;
	
	seek(PLOTTING, 0, 0);
	
	while(my $lineFromData = <PLOTTING>) {
		
		if ($lineFromData =~ m/^set output/) {
			$outputLine = $lineCounter;
		}
		elsif ($lineFromData =~ m/^set yrange/) {
			$yRangeLine = $lineCounter;
		}
		elsif ($lineFromData =~ m/^set xrange/) {
			$xRangeLine = $lineCounter;
		}
		elsif ($lineFromData =~ m/^set ylabel/) {
			$yLabelLine = $lineCounter;
		}
		elsif ($lineFromData =~ m/^set xlabel/) {
			$xLabelLine = $lineCounter;
		}
		elsif ($lineFromData =~ m/^set title/) {
			$titleLine = $lineCounter;
		}		
		$lineCounter++;
		
	}
	
}

sub DrawCreatedVsDelivered() {
	
	@plotFile = @originalPlotFile;
	
	$plotFile[$outputLine] = "set output 'created_X_delivered.png'\n";
	$plotFile[$titleLine] = "set title 'Message Deliverance'\n";
	$plotFile[$xRangeLine] = "set xrange [0:210]\n";
	$plotFile[$yRangeLine] = "set yrange [0:".($maxDeliveredMessages + 50)."]\n";
	$plotFile[$xLabelLine] = "set xlabel 'Created Messages'\n";
	$plotFile[$yLabelLine] = "set ylabel 'Delivered Messages'\n";
	
	my $iterator = 0;
	
	while($iterator < $routersSize)	{
	
		my $actualRouter = $routingType[$iterator];
		my $actualColor = $routingColor[$iterator];
		
		my $plotInstruction = "'delivered_messages_".$actualRouter.".txt' with lines lt 1 lc rgb '".$actualColor."'"; 
		push(@plotFile, $plotInstruction);
		
		if($iterator + 1 != $routersSize) {
			push(@plotFile, ", ");	
		}
		
		$iterator++;
		
	}
	
	open(PLOTTING, ">temp_config.plot") or die "Couldn't create temp_config.plot file.";
	print PLOTTING @plotFile;
	close(PLOTTING);
	
	system("gnuplot temp_config.plot >> plotting_log.txt");
	
}

sub DrawCreatedVsAverageHops() {
	
	@plotFile = @originalPlotFile;
	
	$plotFile[$outputLine] = "set output 'created_X_averagehops.png'\n";
	$plotFile[$titleLine] = "set title 'Average Hops'\n";
	$plotFile[$xRangeLine] = "set xrange [0:210]\n";
	$plotFile[$yRangeLine] = "set yrange [0:".($maxAverageHopsMessages + 2)."]\n";
	$plotFile[$xLabelLine] = "set xlabel 'Created Messages'\n";
	$plotFile[$yLabelLine] = "set ylabel 'Average Hops For Delivered'\n";
	
	my $iterator = 0;
	
	while($iterator < $routersSize)	{
	
		my $actualRouter = $routingType[$iterator];
		my $actualColor = $routingColor[$iterator];
		
		my $plotInstruction = "'created_avghops_".$actualRouter.".txt' with lines lt 1 lc rgb '".$actualColor."'"; 
		push(@plotFile, $plotInstruction);
		
		if($iterator + 1 != $routersSize) {
			push(@plotFile, ", ");	
		}
		
		$iterator++;
		
	}
	
	open(PLOTTING, ">temp_config.plot") or die "Couldn't create temp_config.plot file.";
	print PLOTTING @plotFile;
	close(PLOTTING);
	
	system("gnuplot temp_config.plot >> plotting_log.txt");

}

sub DrawCreatedVsAverageTime() {
	
	@plotFile = @originalPlotFile;
	
	$plotFile[$outputLine] = "set output 'created_X_averagetime.png'\n";
	$plotFile[$titleLine] = "set title 'Average Delay'\n";
	$plotFile[$xRangeLine] = "set xrange [0:210]\n";
	$plotFile[$yRangeLine] = "set yrange [0:".($maxAverageLatencyMessages + 10)."]\n";
	$plotFile[$xLabelLine] = "set xlabel 'Created Messages'\n";
	$plotFile[$yLabelLine] = "set ylabel 'Average Latency Time For Delivered'\n";
	
	my $iterator = 0;
	
	while($iterator < $routersSize)	{
	
		my $actualRouter = $routingType[$iterator];
		my $actualColor = $routingColor[$iterator];
		
		my $plotInstruction = "'created_avgtime_".$actualRouter.".txt' with lines lt 1 lc rgb '".$actualColor."'"; 
		push(@plotFile, $plotInstruction);
		
		if($iterator + 1 != $routersSize) {
			push(@plotFile, ", ");	
		}
		
		$iterator++;
		
	}
	
	open(PLOTTING, ">temp_config.plot") or die "Couldn't create temp_config.plot file.";
	print PLOTTING @plotFile;
	close(PLOTTING);
	
	system("gnuplot temp_config.plot >> plotting_log.txt");

}

sub DrawRadiusVsAverageTime() {
	
	@plotFile = @originalPlotFile;
	
	$plotFile[$outputLine] = "set output 'radius_X_averagetime.png'\n";
	$plotFile[$titleLine] = "set title 'Average Delay on Radius'\n";
	$plotFile[$xRangeLine] = "set xrange [0:55]\n";
	$plotFile[$yRangeLine] = "set yrange [0:".($maxAverageLatencyRadius + 10)."]\n";
	$plotFile[$xLabelLine] = "set xlabel 'Transmission Radius'\n";
	$plotFile[$yLabelLine] = "set ylabel 'Average Latency Time For Delivered'\n";
	
	my $iterator = 0;
	
	while($iterator < $routersSize)	{
	
		my $actualRouter = $routingType[$iterator];
		my $actualColor = $routingColor[$iterator];
		
		my $plotInstruction = "'delay_varying_radius_".$actualRouter.".txt' with lines lt 1 lc rgb '".$actualColor."'"; 
		push(@plotFile, $plotInstruction);
		
		if($iterator + 1 != $routersSize) {
			push(@plotFile, ", ");	
		}
		
		$iterator++;
		
	}
	
	open(PLOTTING, ">temp_config.plot") or die "Couldn't create temp_config.plot file.";
	print PLOTTING @plotFile;
	close(PLOTTING);
	
	system("gnuplot temp_config.plot >> plotting_log.txt");

}

sub DrawRadiusVsProbability() {
	
	@plotFile = @originalPlotFile;
	
	$plotFile[$outputLine] = "set output 'radius_X_probability.png'\n";
	$plotFile[$titleLine] = "set title 'Deliverance Probability on Radius'\n";
	$plotFile[$xRangeLine] = "set xrange [0:55]\n";
	$plotFile[$yRangeLine] = "set yrange [0:1]\n";
	$plotFile[$xLabelLine] = "set xlabel 'Transmission Radius'\n";
	$plotFile[$yLabelLine] = "set ylabel 'Deliverance Probability'\n";
	
	my $iterator = 0;
	
	while($iterator < $routersSize)	{
	
		my $actualRouter = $routingType[$iterator];
		my $actualColor = $routingColor[$iterator];
		
		my $plotInstruction = "'probability_varying_radius_".$actualRouter.".txt' with lines lt 1 lc rgb '".$actualColor."'"; 
		push(@plotFile, $plotInstruction);
		
		if($iterator + 1 != $routersSize) {
			push(@plotFile, ", ");	
		}
		
		$iterator++;
		
	}
	
	open(PLOTTING, ">temp_config.plot") or die "Couldn't create temp_config.plot file.";
	print PLOTTING @plotFile;
	close(PLOTTING);
	
	system("gnuplot temp_config.plot >> plotting_log.txt");

}

sub DrawHostsVsProbability() {
	
	@plotFile = @originalPlotFile;
	
	$plotFile[$outputLine] = "set output 'hosts_X_probability.png'\n";
	$plotFile[$titleLine] = "set title 'Deliverance Probability on Hosts'\n";
	$plotFile[$xRangeLine] = "set xrange [0:55]\n";
	$plotFile[$yRangeLine] = "set yrange [0:1]\n";
	$plotFile[$xLabelLine] = "set xlabel 'Number of Hosts'\n";
	$plotFile[$yLabelLine] = "set ylabel 'Deliverance Probability'\n";
	
	my $iterator = 0;
	
	while($iterator < $routersSize)	{
	
		my $actualRouter = $routingType[$iterator];
		my $actualColor = $routingColor[$iterator];
		
		my $plotInstruction = "'probability_varying_nodes_".$actualRouter.".txt' with lines lt 1 lc rgb '".$actualColor."'"; 
		push(@plotFile, $plotInstruction);
		
		if($iterator + 1 != $routersSize) {
			push(@plotFile, ", ");	
		}
		
		$iterator++;
		
	}
	
	open(PLOTTING, ">temp_config.plot") or die "Couldn't create temp_config.plot file.";
	print PLOTTING @plotFile;
	close(PLOTTING);
	
	system("gnuplot temp_config.plot >> plotting_log.txt");

}

sub DrawGraphics() {
	
	ParsePlottingConfig();
	
	DrawCreatedVsDelivered();
	DrawCreatedVsAverageHops();
	DrawCreatedVsAverageTime();
	DrawRadiusVsAverageTime();
	DrawRadiusVsProbability();
	DrawHostsVsProbability();
	
}

GetFileAttributes();

print "-- Report from simulation --\n\n";
print "Scenario name: ", $scenarioName, "\n";
#IterateUnderRouters();
print "\n-- End of simulation --\n";

print "-- Generating graphic files --\n\n";
DrawGraphics();
print "Graphics generated successfully.\n";

ClearGarbage();
